'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

var _util = require('./util');

var _ = _interopRequireWildcard(_util);

var _constant = require('./constant');

var _virtualDom = require('./virtual-dom');

var store = {};
var renderTreeIntoContainer = function renderTreeIntoContainer(vtree, container, callback, parentContext) {
	if (!vtree) {
		throw new Error('cannot render ' + vtree + ' to container');
	}
	var id = container[_constant.COMPONENT_ID];
	if (store.hasOwnProperty(id)) {
		store[id].updateTree(container.firstChild, vtree, container, parentContext);
	} else {
		container[_constant.COMPONENT_ID] = id = _.getUid();
		container.innerHTML = '';
		vtree.initTree(container, parentContext);
	}
	store[id] = vtree;
	(0, _virtualDom.clearDidMount)();

	var result = null;
	switch (vtree.vtype) {
		case _constant.VNODE_TYPE.ELEMENT:
			result = container.firstChild;
			break;
		case _constant.VNODE_TYPE.COMPONENT:
			result = vtree.map.get(container.firstChild);
			break;
	}

	if (_.isFn(callback)) {
		callback.call(result);
	}

	return result;
};

var render = function render(vtree, container, callback) {
	return renderTreeIntoContainer(vtree, container, callback);
};

exports.render = render;
var unstable_renderSubtreeIntoContainer = function unstable_renderSubtreeIntoContainer(parentComponent, subVtree, container, callback) {
	var context = parentComponent.vtree ? parentComponent.vtree.context : parentComponent.$cache.parentContext;
	return renderTreeIntoContainer(subVtree, container, callback, context);
};

exports.unstable_renderSubtreeIntoContainer = unstable_renderSubtreeIntoContainer;
var unmountComponentAtNode = function unmountComponentAtNode(container) {
	if (!container.nodeName) {
		throw new Error('expect node');
	}
	var id = container[_constant.COMPONENT_ID];
	if (store.hasOwnProperty(id)) {
		store[id].destroyTree(container.firstChild);
		delete store[id];
		return true;
	}
	return false;
};

exports.unmountComponentAtNode = unmountComponentAtNode;
var findDOMNode = function findDOMNode(node) {
	if (node == null) {
		return null;
	}
	if (node.nodeName) {
		return node;
	}
	var component = node;
	// if component.node equal to false, component must be unmounted
	if (_.isFn(component.getDOMNode) && component.$cache.isMounted) {
		return component.getDOMNode();
	}
	throw new Error('findDOMNode can not find Node');
};
exports.findDOMNode = findDOMNode;