'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

var _util = require('./util');

var _ = _interopRequireWildcard(_util);

var _virtualDom = require('./virtual-dom');

var isValidElement = function isValidElement(obj) {
	return obj != null && !!obj.vtype;
};

exports.isValidElement = isValidElement;
var cloneElement = function cloneElement(originElem, props) {
	for (var _len = arguments.length, children = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
		children[_key - 2] = arguments[_key];
	}

	var type = originElem.type;
	var key = originElem.key;
	var ref = originElem.ref;

	props = _.extend({ key: key, ref: ref }, originElem.props, props);
	var vnode = createElement.apply(undefined, [type, props].concat(children));
	if (vnode.ref === originElem.ref) {
		vnode.refs = originElem.refs;
	}
	return vnode;
};

exports.cloneElement = cloneElement;
var createFactory = function createFactory(type) {
	var factory = function factory() {
		for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
			args[_key2] = arguments[_key2];
		}

		return createElement.apply(undefined, [type].concat(args));
	};
	factory.type = type;
	return factory;
};

exports.createFactory = createFactory;
var createElement = function createElement(type, props) {
	for (var _len3 = arguments.length, children = Array(_len3 > 2 ? _len3 - 2 : 0), _key3 = 2; _key3 < _len3; _key3++) {
		children[_key3 - 2] = arguments[_key3];
	}

	var Vnode = undefined;
	switch (true) {
		case _.isStr(type):
			Vnode = _virtualDom.Velem;
			break;
		case _.isComponent(type):
			Vnode = _virtualDom.Vcomponent;
			break;
		case _.isStatelessComponent(type):
			Vnode = _virtualDom.VstatelessComponent;
			break;
		default:
			throw new Error('React.createElement: unexpect type [ ' + type + ' ]');
	}
	var key = null;
	var ref = null;
	var hasRef = false;
	if (props != null) {
		if (!_.isUndefined(props.key)) {
			key = '' + props.key;
			delete props.key;
		}
		if (!_.isUndefined(props.ref)) {
			ref = props.ref;
			delete props.ref;
			hasRef = true;
		}
	}
	var vnode = new Vnode(type, _.mergeProps(props, children, type.defaultProps));
	vnode.key = key;
	vnode.ref = ref;
	if (hasRef && Vnode !== _virtualDom.VstatelessComponent) {
		(0, _virtualDom.handleVnodeWithRef)(vnode);
	}
	return vnode;
};

exports['default'] = createElement;