'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});

var _Component = require('./Component');

var eventNameAlias = {
	onDoubleClick: 'ondblclick'
};
var getEventName = function getEventName(key) {
	key = eventNameAlias[key] || key;
	return key.toLowerCase();
};

var isNotBubble = true;
var notBubbleEvents = {
	onload: isNotBubble,
	onunload: isNotBubble,
	onscroll: isNotBubble,
	onfocus: isNotBubble,
	onblur: isNotBubble,
	onrowexit: isNotBubble,
	onbeforeunload: isNotBubble,
	onstop: isNotBubble,
	ondragdrop: isNotBubble,
	ondragenter: isNotBubble,
	ondragexit: isNotBubble,
	ondraggesture: isNotBubble,
	ondragover: isNotBubble,
	oncontextmenu: isNotBubble
};

var eventTypes = {};

var addEvent = function addEvent(elem, eventType, listener) {
	eventType = getEventName(eventType);
	var isNotBubble = notBubbleEvents[eventType];

	if (isNotBubble) {
		elem[eventType] = listener;
		return;
	}

	var eventStore = elem.eventStore || (elem.eventStore = {});
	eventStore[eventType] = listener;

	if (!eventTypes[eventType]) {
		// onclick -> click
		document.addEventListener(eventType.substr(2), dispatchEvent);
		eventTypes[eventType] = true;
	}

	if (eventType === 'onchange') {
		addEvent(elem, 'oninput', listener);
	}
};

exports.addEvent = addEvent;
var removeEvent = function removeEvent(elem, eventType) {
	eventType = getEventName(eventType);
	var isNotBubble = notBubbleEvents[eventType];

	if (isNotBubble) {
		elem[eventType] = null;
		return;
	}

	var eventStore = elem.eventStore || (elem.eventStore = {});
	delete eventStore[eventType];

	if (eventType === 'onchange') {
		delete eventStore['oninput'];
	}
};

exports.removeEvent = removeEvent;
var dispatchEvent = function dispatchEvent(event) {
	var target = event.target;
	var type = event.type;

	var eventType = 'on' + type;
	var syntheticEvent = undefined;
	_Component.updateQueue.isPending = true;
	while (target) {
		var _target = target;
		var eventStore = _target.eventStore;

		var listener = eventStore && eventStore[eventType];
		if (!listener) {
			target = target.parentNode;
			continue;
		}
		if (!syntheticEvent) {
			syntheticEvent = {};
			syntheticEvent.nativeEvent = event;
			for (var key in event) {
				syntheticEvent[key] = typeof event[key] === 'function' ? event[key].bind(event) : event[key];
			}
		}
		syntheticEvent.currentTarget = target;
		listener.call(target, syntheticEvent);
		target = target.parentNode;
	}
	_Component.updateQueue.batchUpdate();
};