'use strict';

exports.__esModule = true;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _constant = require('./constant');

var _create = require('./create');

var _create2 = _interopRequireDefault(_create);

var _component = require('./component');

var _diff = require('./diff');

var _diff2 = _interopRequireDefault(_diff);

var _util = require('util');

/**
* patch dom
*/
var patch = function patch(node, patches, parent) {
	if (!patches) {
		return node;
	}
	var vnode = patches.vnode;
	var newVnode = patches.newVnode;
	var type = patches.type;
	var childrenType = patches.childrenType;

	var newNode = undefined;
	parent = node ? node.parentNode : parent;
	switch (type) {
		case _constant.CREATE:
			newNode = _create2['default'](newVnode);
			_util.appendChild(parent, newNode);
			break;
		case _constant.REMOVE:
			_util.removeChild(parent, node);
			break;
		case _constant.REPLACE:
			newNode = _create2['default'](newVnode);
			_util.replaceChild(parent, newNode, node);
			break;
		case _constant.PROPS:
			_util.patchProps(node, vnode.props, newVnode.props);
			break;
		case _constant.UPDATE:
			_component.updateComponent(vnode.component, _util.mergeProps(newVnode.props, newVnode.children));
			newVnode.component = vnode.component;
			break;
	}

	switch (childrenType) {
		case _constant.REMOVE:
			while (node.firstChild) {
				_util.removeChild(node, node.firstChild);
			}
			break;
		case _constant.CREATE:
			newVnode.children = _util.mapChildren(patches.newChildren, function (child) {
				return _create.addChild(node, child);
			});
			break;
		case _constant.REPLACE:
			newVnode.children = _util.mapChildren(newVnode.children, function (newChild, i) {
				var patches = _diff2['default'](vnode.children[i], newChild);
				patch(node.childNodes[i], patches, node);
			});
			while (node.childNodes.length > newVnode.children.length) {
				_util.removeChild(node, node.lastChild);
			}
			break;
	}

	return newNode || node;
};

exports['default'] = patch;
module.exports = exports['default'];