'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports.Vtext = Vtext;
exports.Velem = Velem;
exports.VstatelessComponent = VstatelessComponent;
exports.Vcomponent = Vcomponent;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

var _util = require('./util');

var _ = _interopRequireWildcard(_util);

var _constant = require('./constant');

var _Component = require('./Component');

var _createElement = require('./createElement');

var _diff = require('./diff');

var _diff2 = _interopRequireDefault(_diff);

function Vtree(properties) {
	_.extend(this, properties);
}

var noop = _.noop;
var getDOMNode = function getDOMNode() {
	return this;
};
Vtree.prototype = {
	attachRef: function attachRef(refValue) {
		var refKey = this.ref;
		var refs = this.refs;
		var vtype = this.vtype;

		if (!refs || refKey == null || !refValue) {
			return;
		}
		if (refValue.nodeName) {
			// support react v0.13 style: this.refs.myInput.getDOMNode()
			refValue.getDOMNode = getDOMNode;
		}
		if (_.isFn(refKey)) {
			refKey(refValue);
		} else {
			refs[refKey] = refValue;
		}
	},
	detachRef: function detachRef() {
		var refKey = this.ref;
		var refs = this.refs;

		if (!refs || refKey == null) {
			return;
		}
		if (_.isFn(refKey)) {
			refKey(null);
		} else {
			delete refs[refKey];
		}
	},
	updateRef: function updateRef(newVtree, refValue) {
		if (!this.refs) {
			newVtree.attachRef(refValue);
			return;
		}
		if (!newVtree.refs) {
			this.detachRef();
			return;
		}
		if (this.refs !== newVtree.refs) {
			this.detachRef();
			newVtree.attachRef(refValue);
			return;
		}
		var oldRef = this.ref;
		var newRef = newVtree.ref;
		if (newRef == null) {
			this.detachRef();
		} else if (oldRef !== newRef) {
			this.detachRef();
			newVtree.attachRef(refValue);
		}
	},
	updateTree: function updateTree(node, newVtree, parentNode, parentContext) {
		var newNode = node;
		switch ((0, _diff2['default'])(this, newVtree)) {
			case _constant.DIFF_TYPE.CREATE:
				newNode = newVtree.initTree(parentNode, parentContext);
				break;
			case _constant.DIFF_TYPE.REMOVE:
				this.destroyTree(node);
				break;
			case _constant.DIFF_TYPE.REPLACE:
				var $removeNode = removeNode;
				removeNode = noop;
				this.destroyTree(node);
				removeNode = $removeNode;
				newNode = newVtree.initTree(function (newNode) {
					return parentNode.replaceChild(newNode, node);
				}, parentContext);
				break;
			case _constant.DIFF_TYPE.UPDATE:
				newNode = this.update(node, newVtree, parentNode, parentContext);
				break;
		}
		return newNode;
	}
};

function Vtext(text) {
	this.text = text;
}

Vtext.prototype = new Vtree({
	vtype: _constant.VNODE_TYPE.TEXT,
	attachRef: noop,
	detachRef: noop,
	updateRef: noop,
	update: function update(node, nextVtext) {
		if (nextVtext.text !== this.text) {
			node.replaceData(0, node.length, nextVtext.text);
		}
		return node;
	},
	initTree: function initTree(parentNode) {
		var node = document.createTextNode(this.text);
		appendNode(parentNode, node);
		return node;
	},
	destroyTree: function destroyTree(node) {
		removeNode(node);
	}
});

function Velem(type, props) {
	this.type = type;
	this.props = props;
}

var unmountTree = function unmountTree(vtree) {
	if (isValidComponent(vtree)) {
		vtree.destroyTree();
		return false; //ignore mapping children
	} else if (vtree.vtype === _constant.VNODE_TYPE.ELEMENT) {
			vtree.detachRef();
		}
};
var getInnerHTML = function getInnerHTML(props) {
	var innerHTMLObj = props.dangerouslySetInnerHTML;
	return innerHTMLObj && innerHTMLObj.__html;
};
Velem.prototype = new Vtree({
	vtype: _constant.VNODE_TYPE.ELEMENT,
	eachChildren: function eachChildren(iteratee) {
		var children = this.props.children;

		var newChildren = undefined;
		if (this.sorted) {
			_.eachItem(children, iteratee);
			return;
		}
		// the default children often be nesting array, make it flat and cache
		if (_.isArr(children)) {
			newChildren = [];
			_.flattenChildren(children, function (vchild, index) {
				vchild = getVnode(vchild);
				iteratee(vchild, index);
				newChildren.push(vchild);
			});
			this.props.children = newChildren;
			this.sorted = true;
		} else if (!_.isUndefined(children)) {
			children = this.props.children = getVnode(children);
			iteratee(children, 0);
		}
	},
	initTree: function initTree(parentNode, parentContext) {
		var type = this.type;
		var props = this.props;

		var node = document.createElement(type);
		_.setProps(node, props);
		this.eachChildren(function (vchild) {
			vchild.initTree(node, parentContext);
		});
		appendNode(parentNode, node);
		this.attachRef(node);
		return node;
	},
	destroyTree: function destroyTree(node) {
		var childNodes = [];
		for (var i = 0, len = node.childNodes.length; i < len; i++) {
			childNodes.push(node.childNodes[i]);
		}
		this.eachChildren(function (vchild, index) {
			vchild.destroyTree(childNodes[index]);
		});
		//mapTree(this, unmountTree)
		this.detachRef();
		removeNode(node);
	},
	update: function update(node, newVelem, parentNode, parentContext) {
		var props = this.props;

		var newProps = newVelem.props;
		var oldHtml = getInnerHTML(props);
		if (oldHtml == null) {
			var children = !_.isUndefined(props.children) ? props.children : [];
			if (!_.isArr(children)) {
				children = [children];
			}
			var count = 0;
			var childNodes = node.childNodes;
			newVelem.eachChildren(function (newVchild, index) {
				count += 1;
				var vchild = children[index];
				if (vchild) {
					vchild.updateTree(childNodes[index], newVchild, node, parentContext);
				} else {
					newVchild.initTree(node, parentContext);
				}
			});
			var childrenLen = children.length;
			// destroy old children not in the newChildren
			while (childrenLen > count) {
				children[count].destroyTree(childNodes[count]);
				count += 1;
			}
			_.patchProps(node, props, newProps);
		} else {
			_.patchProps(node, props, newProps);
			newVelem.eachChildren(function (newVchild) {
				return newVchild.initTree(node, parentContext);
			});
		}
		this.updateRef(newVelem, node);
		return node;
	}
});

function VstatelessComponent(type, props) {
	this.type = type;
	this.props = props;
	this.map = new _.Map();
}

VstatelessComponent.prototype = new Vtree({
	vtype: _constant.VNODE_TYPE.STATELESS_COMPONENT,
	attachRef: noop,
	detachRef: noop,
	updateRef: noop,
	renderTree: function renderTree(parentContext) {
		var factory = this.type;
		var props = this.props;

		var componentContext = getContextByTypes(parentContext, factory.contextTypes);
		var vtree = factory(props, componentContext);
		if (vtree && vtree.render) {
			vtree = vtree.render();
		}
		return getVnode(vtree);
	},
	initTree: function initTree(parentNode, parentContext) {
		var vtree = this.renderTree(parentContext);
		var node = vtree.initTree(parentNode, parentContext);
		this.map.set(node, vtree);
		return node;
	},
	destroyTree: function destroyTree(node) {
		var vtree = this.map.remove(node);
		vtree.destroyTree(node);
	},
	update: function update(node, newVstatelessComponent, parentNode, parentContext) {
		var vtree = this.map.remove(node);
		var newVtree = newVstatelessComponent.renderTree(parentContext);
		var newNode = vtree.updateTree(node, newVtree, parentNode, parentContext);
		newVstatelessComponent.map.set(newNode, newVtree);
		return newNode;
	}
});

var setRefs = noop;
var handleVnodeWithRef = function handleVnodeWithRef(vnode) {
	setRefs(vnode);
};
exports.handleVnodeWithRef = handleVnodeWithRef;
var getContextByTypes = function getContextByTypes(curContext, contextTypes) {
	var context = {};
	if (!contextTypes || !curContext) {
		return context;
	}
	for (var key in contextTypes) {
		if (contextTypes.hasOwnProperty(key)) {
			context[key] = curContext[key];
		}
	}
	return context;
};

exports.getContextByTypes = getContextByTypes;
var bindRefs = function bindRefs(refs) {
	return function (vnode) {
		vnode.refs = vnode.refs || refs;
	};
};

var renderComponent = function renderComponent(component, parentContext) {
	setRefs = bindRefs(component.refs);
	var vtree = component.render();
	if (_.isUndefined(vtree)) {
		throw new Error('component can not render undefined');
	}
	vtree = getVnode(vtree);
	var curContext = component.getChildContext();
	if (curContext) {
		curContext = _.extend({}, parentContext, curContext);
	} else {
		curContext = parentContext;
	}
	vtree.context = curContext;
	setRefs = noop;
	return vtree;
};

exports.renderComponent = renderComponent;
var didMountComponents = [];
var callDidMount = function callDidMount(store) {
	return store.vtree.didMount(store.node);
};
var clearDidMount = function clearDidMount() {
	var components = didMountComponents;
	if (components.length === 0) {
		return;
	}
	didMountComponents = [];
	_.eachItem(components, callDidMount);
};

exports.clearDidMount = clearDidMount;
var neverUpdate = function neverUpdate() {
	return false;
};

function Vcomponent(type, props) {
	this.type = type;
	this.props = props;
	this.map = new _.Map();
}

Vcomponent.prototype = new Vtree({
	vtype: _constant.VNODE_TYPE.COMPONENT,
	initTree: function initTree(parentNode, parentContext) {
		var Component = this.type;
		var props = this.props;

		var componentContext = getContextByTypes(parentContext, Component.contextTypes);
		var component = new Component(props, componentContext);
		var updater = component.$updater;
		var cache = component.$cache;

		cache.parentContext = parentContext;
		updater.isPending = true;
		component.props = component.props || props;
		component.componentWillMount();
		(0, _Component.updatePropsAndState)(component, component.props, updater.getState(), component.context);
		var vtree = component.vtree = renderComponent(component, parentContext);
		var node = vtree.initTree(parentNode, vtree.context);
		component.$node = node;
		cache.isMounted = true;
		didMountComponents.push({ node: node, vtree: this });
		this.map.set(node, component);
		cache.map = this.map;
		return node;
	},
	didMount: function didMount(node) {
		var component = this.map.get(node);
		if (!component) {
			return;
		}
		var updater = component.$updater;
		component.componentDidMount();
		updater.isPending = false;
		this.attachRef(component);
		updater.emitUpdate();
	},
	destroyTree: function destroyTree(node) {
		var component = this.map.remove(node);
		if (!component) {
			return;
		}
		this.detachRef();
		component.setState = noop;
		component.componentWillUnmount();
		component.vtree.destroyTree(node);
		delete component.setState;
		component.$cache.isMounted = false;
		component.vtree = component.$node = component.refs = component.context = null;
	},
	update: function update(node, newVtree, parentNode, parentContext) {
		var component = this.map.remove(node);
		if (!component) {
			return;
		}
		var Component = newVtree.type;
		var nextProps = newVtree.props;
		var updater = component.$updater;
		var cache = component.$cache;

		var componentContext = getContextByTypes(parentContext, Component.contextTypes);
		cache.parentContext = parentContext;
		updater.isPending = true;
		component.componentWillReceiveProps(nextProps, componentContext);
		updater.isPending = false;
		newVtree.map.set(node, component);
		cache.map = newVtree.map;
		updater.emitUpdate(nextProps, componentContext);
		var newNode = component.$node;
		this.updateRef(newVtree, component);
		return newNode;
	}
});

var removeNode = function removeNode(node) {
	// if node.parentNode had set innerHTML, do nothing
	if (node && node.parentNode) {
		node.parentNode.removeChild(node);
	}
};
var appendNode = function appendNode(parentNode, node) {
	// for replacing node
	if (_.isFn(parentNode)) {
		parentNode(node);
	} else {
		parentNode.appendChild(node);
	}
};

var mapTree = function mapTree(tree, iteratee) {
	var queue = [tree];
	while (queue.length) {
		var item = queue.shift();
		// as you know, vnode's children may be nested list
		// but we only map old tree which has been sorted
		// if (_.isArr(item)) {
		// 	queue.splice(0, 0, ...item)
		// 	continue
		// }
		// if iteratee return false, ignore mapping children
		if (iteratee(item) === false) {
			continue;
		}
		if (item && item.props && !_.isUndefined(item.props.children)) {
			_.isArr(item.props.children) ? queue.push.apply(queue, _toConsumableArray(item.props.children)) : queue.push(item.props.children);
		}
	}
};

var getVnode = function getVnode(vnode) {
	if (vnode === null) {
		vnode = new Velem('noscript', {});
	} else if (!(0, _createElement.isValidElement)(vnode)) {
		vnode = new Vtext(vnode);
	}
	return vnode;
};

var isValidComponent = function isValidComponent(obj) {
	if (obj == null) {
		return false;
	}
	var vtype = obj.vtype;
	if (vtype === _constant.VNODE_TYPE.COMPONENT || vtype === _constant.VNODE_TYPE.STATELESS_COMPONENT) {
		return true;
	}
	return false;
};