'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _util = require('./util');

var _constant = require('./constant');

/**
* diff vnode and newVnode
*/
var diff = function diff(vnode, newVnode) {
	var children = undefined;
	var type = undefined;
	switch (true) {
		case vnode === newVnode:
			return;
		case newVnode == null:
			type = _constant.REMOVE;
			break;
		case vnode == null:
			type = _constant.CREATE;
			break;
		case (_util.isStr(vnode) || _util.isStr(newVnode)) && vnode !== newVnode:
			type = _constant.REPLACE;
			break;
		case vnode.tagName !== newVnode.tagName:
			type = _constant.REPLACE;
			break;
		case vnode.type === _constant.WIDGET && newVnode.type === _constant.WIDGET:
			type = _constant.WIDGET;
			break;
		case vnode.type === _constant.WIDGET || newVnode.type === _constant.WIDGET:
			type = _constant.REPLACE;
			break;
		case !!(vnode.props || newVnode.props):
			type = _constant.PROPS;
			break;
	}
	if (!type || type === _constant.PROPS) {
		children = diffChildren(vnode.children, newVnode.children);
		if (children) {
			return _extends({ type: type, vnode: vnode, newVnode: newVnode }, children);
		}
	}

	return type ? { type: type, vnode: vnode, newVnode: newVnode } : null;
};

exports['default'] = diff;

var diffChildren = function diffChildren(children, newChildren) {
	children = getFlatChildren([], children);
	newChildren = getFlatChildren([], newChildren);
	var childrenType = undefined;
	var childrenPatches = undefined;
	if (children.length === 0) {
		if (newChildren.length > 0) {
			childrenType = _constant.CREATE;
		}
	} else if (newChildren.length === 0) {
		childrenType = _constant.REMOVE;
	} else {
		childrenPatches = [];
		var maxLen = Math.max(children.length, newChildren.length);
		for (var i = 0; i < maxLen; i++) {
			childrenPatches.push(diff(children[i], newChildren[i]));
		}
		childrenType = _constant.REPLACE;
		return { childrenType: childrenType, childrenPatches: childrenPatches };
	}

	if (childrenType) {
		return { childrenType: childrenType, newChildren: newChildren };
	}
};

var getFlatChildren = function getFlatChildren(store, children) {
	if (_util.isArr(children)) {
		children.forEach(function (item) {
			if (_util.isArr(item)) {
				return getFlatChildren(store, item);
			}
			store.push(item);
		});
	}
	return store;
};
module.exports = exports['default'];