'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _util = require('./util');

var _constant = require('./constant');

var _create = require('./create');

var _create2 = _interopRequireDefault(_create);

/**
* patch dom
*/
var patch = function patch(node, patches, parent) {
	if (!patches) {
		return node;
	}
	var vnode = patches.vnode;
	var newVnode = patches.newVnode;
	var type = patches.type;
	var childrenType = patches.childrenType;

	var newNode = undefined;
	parent = node ? node.parentNode : parent;
	switch (type) {
		case _constant.CREATE:
			newNode = _create2['default'](newVnode);
			parent.appendChild(newNode);
			break;
		case _constant.REMOVE:
			parent.removeChild(node);
			break;
		case _constant.REPLACE:
			newNode = _create2['default'](newVnode);
			parent.replaceChild(newNode, node);
			break;
		case _constant.PROPS:
			applyProps(node, vnode.props, newVnode.props);
			break;
		case _constant.WIDGET:
			newVnode.update(vnode, node);
			break;
	}

	switch (childrenType) {
		case _constant.REMOVE:
			node.innerHTML = '';
			break;
		case _constant.CREATE:
			patches.newChildren.forEach(function (child) {
				return _create.addChild(node, child);
			});
			break;
		case _constant.REPLACE:
			var children = Array.prototype.slice.call(node.childNodes);
			patches.childrenPatches.forEach(function (childPatches, index) {
				patch(children[index], childPatches, node);
			});
			break;
	}

	return newNode || node;
};

exports['default'] = patch;

var applyProps = function applyProps(node, props, newProps) {
	if (props == null && _util.isObj(newProps)) {
		return _util.setProps(node, newProps);
	} else if (newProps == null && _util.isObj(props)) {
		return Object.keys(props).each(function (key) {
			return removeProp(node, key);
		});
	}
	Object.keys(_extends({}, props, newProps)).forEach(function (key) {
		var value = props[key];
		var newValue = newProps[key];
		switch (true) {
			case key === 'style':
				patchStyle(node, props.style, newProps.style);
				break;
			case _util.isEventKey(key):
				if (!_util.isFn(newValue)) {
					_util.removeEvent(node, key);
				} else if (newValue !== value) {
					_util.setEvent(node, key, newValue);
				}
				break;
			case key in node:
				if (newValue === undefined) {
					removeProp(node, key);
				} else if (newValue !== value) {
					node[key] = newValue;
				}
				break;
			default:
				if (newValue === undefined) {
					node.removeAttribute(key);
				} else {
					node.setAttribute(key, newValue);
				}
		}
	});
};

var patchStyle = function patchStyle(node, style, newStyle) {
	var domStyle = node.style;
	Object.keys(_extends({}, style, newStyle)).forEach(function (key) {
		var value = newStyle[key];
		if (value === undefined) {
			domStyle[key] = '';
		} else if (value !== style[key]) {
			_util.setStyleValue(domStyle, key, value);
		}
	});
};
module.exports = exports['default'];